function [rootOut, weightOut, levOut, mulOut, cartan, metric, ...
  invcartan] = lietyperank2rootsweights( Control, type, rank, dynkin)
%LIETYPERANK2ROOTSWEIGHTS  calculate roots from Lie algebra type/rank
%
%    [root, wt, lev, mul, cartan, metric, ...
%      invcartan] = lietyperank2rootsweights( Control, rank, type, dynkin)
%
%  rank      : rank for lie algebra
%  type      : type of Lie algebra (type = 'a','b','c','d','e','f','g')
%  dynkin(j) : Dynkin indices (j=1,...,rank)
%
%  wt(i,j) : j'th Dynkin index of the i'th weight
%  lev(i)  : level of i'th weight
%  mul(i)  : multiplicity of i'th weight
%
%  Source:
%    Vishnu K. Agrawala and Johan G. Belinfante:
%      Weight diagrams for Lie group representations: a computer 
%      implementation algorithm of Freudenthal's algorithm in ALGOL 
%      and FORTRAN, BIT, 9 (1969), 301-314

% Copyright (c) V. K Agrawala 1969
% Copyright (c) G. Beyerle 2011-2012
%
% This program comes with ABSOLUTELY NO WARRANTY. This is free software, 
% and you are welcome to redistribute it under certain conditions;

%
% notes: 
%
% - weights are listed in different order compared to Agrawala (1969)
% - this implementation uses nested function because the ALGOL
%   source in Agrawala (1969) relies on this feature.
%

% examples:

  if 0,
    c = [];
    rank   = 2;
    type   = 'a';
    dynkin = [1 0];
    [root, wt, lev, mul] = lietyperank2rootsweights( c, type, rank, dynkin);
  end
  
  if 0,
    c = [];
    rank   = 2;
    type   = 'a';
    dynkin = [3 3];
    [root, wt, lev, mul] = lietyperank2rootsweights( c, type, rank, dynkin);
  end
  
  if 0,
    c = [];
    rank   = 2;
    type   = 'g';
    dynkin = [3 3];
    [root, wt, lev, mul] = lietyperank2rootsweights( c, type, rank, dynkin);
  end
  
  if 0,
    c = [];
    rank   = 3;
    type   = 'a';
%    dynkin = [3 3 3];
    dynkin = [1 1 1];
    [root, wt, lev, mul] = lietyperank2rootsweights( c, type, rank, dynkin);
  end
  
  if 0,
    c = [];
    rank   = 4;
    type   = 'd';
    dynkin = [1 0 1 0];
    [root, wt, lev, mul] = lietyperank2rootsweights( c, type, rank, dynkin);
  end

%  disp( [upper( mfilename) ': TODO - sort roots according to size!'])
 
  if nargin < 4,
    dynkin = zeros( 1, rank);
  end

  local_validateinput( type, rank);
  
  Control = local_initcontrol( Control);
  
  rank   = int32( rank);
  dynkin = int32( dynkin);

% -------------------------------------------------------------------------
%
%  reduce integer fractions to lowest terms
%
%  given the integers a and b, find the intergers c and d
%  such that
%
%    c / d = a / b
%
%  where c and d are without common factor
%
%  (gcd: greatest common denominator)
%
  function [c, d] = nested_reduce(a, b)

    aa = abs( a);
    bb = abs( b);

    if a == 0
      gcd = bb;
    elseif b == 0
      gcd = aa;    
    else
      q = idivide( aa, bb);
      while (aa ~= bb * q)
        r = aa - bb * q;
        aa = bb;
        bb = r;
        q = idivide( aa, bb);
      end
      gcd = bb;
    end
    c = idivide( a, gcd);
    d = idivide( b, gcd);
    if d < 0
      c = -c;
      d = -d;
    end

  end

% -------------------------------------------------------------------------
%
%  reduce integer fractions to lowest terms
%
%  given the integers a, b, c and d find the 
%  intergers p, q, r, s, num and den such that
%
%    p / q     = a / b
%    r / s     = c / d
%    num / den = p*r / (q*s)
%
%  where p and q, r and s, num and den are without 
%  a common factor
%
%  (gcd: greatest common denominator)
%
  function [num, den] = nested_ratmul( a, b, c, d)

    [p, q]     = nested_reduce( a, b);
    [r, s]     = nested_reduce( c, d);
    
    [num, den] = nested_reduce( p*r, q*s);

  end

% -------------------------------------------------------------------------
%
%  determine weight/root diagrams
%
  function [numwts, wt, lev] = nested_weightdiagram( rank, dynkin)

%    keyboard
    
%     function ret = nested_generatehash( w)
%       ret = sum( hashval .* double( w));
%     end

% %
% %  look up weight w in list of previously determined weights wt 
% %
%     function belong = nested_belong_sav( wt, w)
% %    function belong = nested_belong( wt, w)
%       belong = 0;
%       for n = 1:numwts,
% %         for i = 1:rank,
% %           if w(i) == wt(n,i)
% %             belong = 1;
% %             return;
% %           end
% %         end
%         if all( w == wt(n,:))
%           belong = 1;
%           return;
%         end
% %         if min( abs( w - wt(n,:))) == 0
% %           belong = 1;
% %           return;
% %         end
%       end
%     end % --- nested_belong() ---

% %
% %  look up weight w in list of previously determined weights wt 
% %
%     function belong = nested_belong_sav2( wt, w)
%             
%       delta = sum( abs( repmat( w, numwts, 1) - wt(1:numwts,:)), 2);
%       if min( delta) == 0,
%         belong = 1;
%       else
%         belong = 0;
%       end
%       
%     end % --- nested_belong() ---

%
%  look up weight w in list of previously determined weights wt 
%
    function belong = nested_belong( wt, w)
            
      delta = sum( abs( w(ones(1,numwts),:) - wt(1:numwts,:)), 2);
      if min( delta) == 0,
        belong = 1;
      else
        belong = 0;
      end
      
    end % --- nested_belong() ---

% %
% %  look up weight w in list of previously determined weights wt 
% %
%     function belong = nested_belong_hash( wt, w)            
%       belong = any( wthash == nested_generatehash( w));       
%     end % --- nested_belong() ---

%
%  allocate memory
%
%    wt  = zeros( MaxNofWeights, rank, 'int32');
    wt     = zeros( MaxNofWeights, rank, 'int32');
    lev    = zeros( MaxNofWeights, 1, 'int32');
    wthash = zeros( MaxNofWeights, 1, 'int32');

%    hashval = 4.^(0:double(rank)-1);
    
    maxlevel = detcar;
    wt(1,:)  = dynkin;
    
%    wthash(1) = nested_generatehash( dynkin); 
    
    for i = 1:rank,
%      wt(1,i) = dynkin(i);
%       for j = 1:rank, 
%         maxlevel = maxlevel + 2 * carinv(i,j) * dynkin(j);
%       end
      maxlevel = maxlevel + sum( 2 * carinv(i,:) .* dynkin);
    end  
%    maxlevel
    maxlevel = idivide( maxlevel, detcar);
    
%    disp( [ upper( mfilename) ': max. level = ' num2str( maxlevel)])
    
    numwts = 1;
    first  = 1;
    last   = 1;
    lev(1) = 1;

%     maxlevel = 2
%     keyboard

    tic
    for level = (2:maxlevel),
      
      for parent = (first:last),
                
        for jthrt = (rank:-1:1),

          wj = wt(parent,jthrt);
          
%          wj      
%           for i = 1:rank,
%             w(i)  = wt(parent,i) - cartan(i,jthrt);
%             ww(i) = wt(parent,i) + (1-wj) * cartan(i,jthrt);
%           end

%
%  cartan(:,j) is the j-th simple/fundamental root 
%
          w  = wt(parent,:) - cartan(:,jthrt).';
          
%           if all( w == 0)
%             wt(parent,:)
%             cartan(:,jthrt).'
%             keyboard
%           end
          
          ww = wt(parent,:) + (1-wj) * cartan(:,jthrt).';
          
%           parent
%           jthrt
%           wj
%           w
%           ww
%         priv_assert( nested_belong( wt, w) == nested_belong_new( wt, w))

%  check if we've been here before
          if nested_belong( wt, w),
            continue
          end
          
 %         priv_assert( nested_belong( wt, ww) == nested_belong_new( wt, ww))

          if wj <= 0 && ~nested_belong( wt, ww),
            continue; % go to next root/weight
          end
          
          numwts = numwts + 1;
          
          if numwts > MaxNofWeights
            error( [upper( mfilename) ...
              ': numwts exceeds hard-coded limit, increase parameter MaxNofWeights!'])
          end
   
% new weight add to list 'wt'          
          lev(numwts) = level;
%           for i = 1:rank,
%             wt(numwts,i) = w(i);
%           end
          wt(numwts,:) = w;
%          wthash(numwts) = nested_generatehash( w); 
        end
        
      end  % --- for parent = first:last, ---

%      wt
%      keyboard
      
      if numwts == last,          
        return
      end
      
%      keyboard
      
      first = last + 1;
      last  = numwts;
      
      for p = first:last,
        for q = p+1:last,
          
          for j = 1:rank,
%             sum = 0;
%             for i = 1:rank,
%               sum = sum + carinv(j,i) * (wt(p,i) - wt(q,i));
%             end
            sum2 = sum( carinv(j,:) .* (wt(p,:) - wt(q,:)));
            
            if sum2 == 0,
              continue;
            end
            
%
%  swap wt(p,:) and wt(q,:) if angle between
%
%    carinv(j,:)  
%
%  and  
%
%    (wt(p,:) - wt(q,:))            
%
%  exceeds 90 degrees.
%

            if sum2 < 0,
% swap wt(p,:) and wt(q,:) 
%              for i = 1:rank,
%                 x = wt(p,i);
%                 wt(p,i) = wt(q,i);
%                 wt(q,i) = x;
%              end
              tmp       = wt(p,:);
              wt(p,:)   = wt(q,:);
              wt(q,:)   = tmp;
              tmp       = wthash(p);
              wthash(p) = wthash(q);
              wthash(q) = tmp;
              break; % back for j ... loop to for q ... loop
            end 
            
          end   % --- for j = ...
        end   % --- for q = ... 
      end   % --- for p = ... 
    end   % ---  for level = 2:maxlevel, ---

    ElapsedTime = toc;
    disp( [upper( mfilename) ': timing nested_weightdiagram() >' num2str( ElapsedTime) ' sec<'])
  
  end

% -------------------------------------------------------------------------
% 
%  Multiplicity of highest weight of an irreducible representation
%  is unity. Multiplicity of any lower weight 'mu' may be computed by the
%  Freudenthal recursion relation
%  
  function [numwts, wt, lev, mul] = nested_freudenthal( rank, dynkin)

    mul  = zeros( MaxNofWeights, 1, 'int32');

    [numwts, wt, lev] = nested_weightdiagram( rank, dynkin);
    mul(1) = 1;

    if numwts == 1,
      return
    end
    
%    beta = zeros( 1, rank, 'int32');
    
    for nthwt = 2:numwts
%       for i = 1:rank,
%         w(i) = wt(nthwt,i);
%       end
      w = wt(nthwt,:);
      left = int32( 0);

      for k = 1:rank,
%         for j = 1:rank,
%           left = left + metric(k,j) * ((dynkin(k)+1) * (dynkin(j)+1) - ...
%                                   (w(k)+1) * (w(j)+1));
%         end
        left = left + sum( metric(k,:) .* ...
            ((dynkin(k) + 1) .* (dynkin + 1) - ...
             (w(k) + 1) .* (w + 1)));
      end
      
      if left == 0
        mul(nthwt) = 0; 
        continue;  % next weight
      end
      
      right = int32( 0);
      
      for x = 1:numposrts,
        jj = 0;
        
        loop = 1;
        while loop,
        
          jj = jj + 1;
%           for i = 1:rank,
%             beta(i) = w(i) + jj * root(x,i);
%           end        
          beta = w + jj * root(x,:);
          
          for beth = 1:nthwt

%             for i = 1:rank,
%               if wt(beth,i) ~= beta(i)
%                 break; 
%               end
%             end

%            if any( wt(beth,:) ~= beta)
            if any( wt(beth,:) - beta)
              continue;
            end

            for j = 1:rank,
%               for k = 1:rank,
%                 right = right + ...
%                   2 * mul(beth) * metric(j,k) * root(x,j) * beta(k);
%               end          
              right = right + ...
                sum( 2 * mul(beth) * metric(j,:) .* root(x,j) .* beta);
            end
            
            loop = -1;   % jump to jj = jj + 1;
          end
          if loop < 0,
            loop = 1;
          else
            loop = 0;  % leave while loop
          end
        end  % --- for beth = 1:nthwt
          
      end
      mul(nthwt) = idivide( right, left);
    end
    
  end   % --- nested_freudenthal() ---

%
%  hard coded limit for maximum number of weights
%
  MaxNofWeights = 10000;

  [cartan, carinv, adjdynkin, metric, dimalg, detcar, ...
    m, mden] = local_initializevars( rank, type);
    
  if length( dynkin) ~= rank,
    error( [ upper( mfilename) ...
      ': number of Dynkin indices (' num2str( length( dynkin)) ...
      ') differs from rank (' num2str( rank) ').'])
  end

%
%  number of roots (here, zero roots are counted as one root!)
%  numroots : number of pos. roots + 
%             number of neg. roots + 1 
%
  numroots = dimalg - rank + 1;
  disp( [upper( mfilename) ': number of roots             = ' ...
    num2str( numroots-1)])

  disp( [upper( mfilename) ': dimension of Lie algebra    = ' ...
    num2str( dimalg)])

  %
  %  convert to integers
  %
  dimalg = int32( dimalg);
  detcar = int32( detcar);
  mden   = int32( mden);

%   a = int32( 5);
%   for b = int32( 1:17),
%     [c, d] = nested_reduce( a, b)
%   end
%   keyboard
  
  gden      = mden * detcar;
  
%
%  numposrts : number of positive roots
%
  numposrts = idivide( dimalg - rank, int32( 2));
  
  disp( [upper( mfilename) ': number of positive roots    = ' ...
    num2str( numposrts)])
  
%
%  determine inverse of Cartan matrix 'carinv'
%  (make sure, entries in 'carinv' are integers.)
%
  
  for i = 1:rank,
    for j = 1:rank,
%    for j = i+1:rank,
      m(j,i) = idivide( m(i,i) * cartan(i,j), int32( 2));
%      m(j,i) = m(i,i) * idivide( cartan(i,j), int32( 2));
      m(i,j) = m(j,i);
      carinv(i,j) = idivide( int32( 2) * metric(i,j), m(i,i));
%      carinv(i,j) = int32( 2) * idivide( metric(i,j), m(i,i));
    end
  end
   
%   for i = 1:rank,
%     for j = 1:rank,
%       tmp(i,j) = sum( cartan(i,:) .* carinv(j,:));
%     end
%   end
%   tmp
%   cartan
%  carinv

%
%  check for consistency, numerical errors
%
  Tmp = inv( double( cartan)) * double( detcar) - double( carinv);
  if ( max( abs( Tmp(:))) > 1e-6)
    
    double( carinv)
    inv( double( cartan)) * double( detcar)
    cartan
    metric
    m
    Tmp
%    carinv = int32( inv( double( cartan)) * double( detcar))
    error( [ upper( mfilename) ...
      ': numerical errors in calculating the inverse Cartan matrix']);
    keyboard
  end
      
  priv_assert( max( abs( Tmp(:))) < 1e-6)

  invcartan = carinv;
   
%
%  no need to use the Freudenthal algorithm since
%  all non-zero roots have multiplicity one,
%  zero roots have multiplicity equal to rank of algebra
%
  
%
%  The roots are the weights of the adjoint representation.
%  Thus, in order to get the roots we calculate the weights
%  for Dynkin index of adjoint representation 
%  (Dynkin indices set equal to adjdynkin).
%
%   adjdynkin
%   keyboard
  [tmp, root, adjlev] = nested_weightdiagram( rank, adjdynkin);
  
%   root(1:tmp + (rank-1),:)
%   keyboard
  
%  whos
%  tmp
%  keyboard
  
%   nested_weightdiagram counts just one zero root 
%   tmp : number of roots with zero roots just counted once
%   (number of zero roots is 'rank')
%

%  numroots = tmp + (rank-1);
  
  root(numroots+1:end,:) = [];
  adjlev(numroots+1:end) = [];

%   tic
%   [adnumwts, adwt, adjlev, adjmul] = nested_freudenthal( rank, adjdynkin);
%   toc

  adjmul = ones( size( adjlev));
  adjmul(all( root' == 0)) = rank;
  
  priv_assert( sum( adjmul) == dimalg)
  
  if Control.Verbose
    disp( ' ');
  %  disp( ' root no   level   multiplicity  Dynkin index ');
    disp( ' root no   level   multiplicity   root vector  ');
    for i = 1:numroots
      disp( [sprintf( ['  %3d        %2d         %2d        [%2d' ...
        repmat( ',%2d', 1, rank-1) ']'], ...
        i, adjlev(i), adjmul(i), root(i,:))])
    end
    disp( ' ');
  end  
  
%   adjlev
%   root
%   tmp
%   numroots
%   keyboard
  
%  whos
%  tmp
%  keyboard

  
%   root
%   if rank == 2
%     clfig(1)
%     plot( root(:,1), root(:,2), 'kp')
%   end
%   keyboard
  
  r = int32( 1);
  s = int32( 1);
  
  for x = 1:numposrts,
    
    p = int32( 0);
    q = int32( 0);
%     p1 = int32( 0);
%     q1 = int32( 0);
    
    for i = 1:rank,
%        for j = 1:rank,
%          p = p + metric(i,j) * root(x,i) * (dynkin(j)+1);
%          q = q + metric(i,j) * root(x,i);
%        end
      p = p + sum( metric(i,:) .* root(x,i) .* (dynkin+1));
      q = q + sum( metric(i,:) * root(x,i));
    end
    
%     priv_assert( p1 == p)
%     priv_assert( q1 == q)
    
%     [u, v] = nested_ratmul( p, q, r, s);
%     r = u;
%     s = v;
     [r, s] = nested_ratmul( p, q, r, s);
    
%     [p, q, r, s]
     
  end
  
%  dimrep = r;
%  disp( [ upper( mfilename) ': dimrep = ' num2str( dimrep)])

  p = int32( 0);
  q = gden;
  
  for i = 1:rank,
%     for j = 1:rank,
%       p = p + metric(i,j) * dynkin(i) * (dynkin(j)+2);      
%     end
    p = sum( metric(i,:) .* dynkin(i) .* (dynkin + 2));      
  end
  
  [casnum, casden] = nested_reduce( p, q);

%   [adnumwts, adwt, adjlev, adjmul] = nested_freudenthal( rank, adjdynkin);
%   adwt(1:adnumwts,:)
%   adjlev(1:adnumwts)
%   adjmul(1:adnumwts)
%   keyboard
  
%  tic
  [numwts, wt, lev, mul] = nested_freudenthal( rank, dynkin);
%  toc
  
  wt(numwts+1:end,:) = [];
  lev(numwts+1:end)  = [];
  mul(numwts+1:end)  = [];
    
%  metric = metric;
  
  rootOut   = root';  % transpose!
  weightOut = wt';    % transpose!
  levOut    = lev';   % transpose!
  mulOut    = mul';   % transpose!
  
%
%  consistency checks
%
  local_testresults( rootOut, cartan, metric, rank);

  disp( [ upper( mfilename) ': number of weights           = ' ...
      num2str( length( mul))])
  disp( [ upper( mfilename) ': dimension of representation = ' ...
      num2str( sum( mul))])
  disp( [ upper( mfilename) ': maximum level               = ' ...
      num2str( max( lev))])

  if rank == 1
    if Control.DoPlot
      clfig(1)
      h=plot( root(:,1), 'rp');
      set(h,'markersize',10)
      hold on
      h=plot( wt(:,1), 'ko');
      set(h,'markersize',5)
    end
  end
  
%
%  plot result
%
  if Control.DoPlot
    if rank == 2
      clfig(1)
      tmpRX = double(root(:,1));      
      tmpRY = double(root(:,2));      
      h=plot( tmpRX, tmpRY, 'rp');
      set(h,'markersize',10)
      hold on
      for n=1:size( root, 1)
        text( tmpRX(n)+0.05, tmpRY(n), num2str( adjlev(n)), ...
          'horizontalal', 'left', 'verticalal', 'top', 'fontsize', 14 )
      end
      xMax = max( tmpRX); 
      yMax = max( tmpRY); 
      set( gca, 'xlim', [-xMax-0.5 xMax+0.5])
      set( gca, 'ylim', [-yMax-0.5 yMax+0.5])

      clfig(2)
      tmpWX = double(wt(:,1));      
      tmpWY = double(wt(:,2));      
%      h=plot( root(:,1), root(:,2), 'rp');
      h=plot( tmpRX, tmpRY, 'rp');
      set(h,'markersize',10)
      hold on
%      h=plot( wt(:,1), wt(:,2), 'ko');
      h=plot( tmpWX, tmpWY, 'ko');
      set(h,'markersize',5)
      xMax = max( [max( tmpRX), max( tmpWX)]); 
      yMax = max( [max( tmpRY), max( tmpWY)]); 
%       xMax = max( tmpRX); 
%       yMax = max( tmpRY); 
      set( gca, 'xlim', [-xMax-0.5 xMax+0.5])
      set( gca, 'ylim', [-yMax-0.5 yMax+0.5])

      clfig(3)
      tmpRX = double(root(:,1));      
      tmpRY = double(root(:,2)) + tmpRX/2;      
%      h=plot( tmp(:,1), tmp(:,2) + double(tmp(:,1))/2, 'rp');
      h=plot( tmpRX, tmpRY, 'rp');
      set(h,'markersize',10)
      hold on
%      tmp = double(wt);
      tmpWX = double(wt(:,1));      
      tmpWY = double(wt(:,2)) + tmpWX/2;      
%      h=plot( tmp(:,1), tmp(:,2) + double(tmp(:,1))/2, 'ko');
      h=plot( tmpWX, tmpWY, 'ko');
      set(h,'markersize',5)
      for n=1:size( root, 1)
        text( tmpRX(n)+0.05, tmpRY(n), num2str( adjlev(n)), ...
          'horizontalal', 'left', 'verticalal', 'top', 'fontsize', 14 )
      end
      xMax = max( [max( tmpRX), max( tmpWX)]); 
      yMax = max( [max( tmpRY), max( tmpWY)]); 
      set( gca, 'xlim', [-xMax-0.5 xMax+0.5])
      set( gca, 'ylim', [-yMax-0.5 yMax+0.5])
  %    keyboard
    end
  
    if rank == 3
      clfig(1)
      h=plot3( root(:,1), root(:,2), root(:,3), 'rp');
      set(h,'markersize',10)

      clfig(3)
      h=plot3( root(:,1), root(:,2), root(:,3), 'rp');
      set(h,'markersize',10)
      hold on
      h=plot3( wt(:,1), wt(:,2), wt(:,3), 'ko');
      set(h,'markersize',5)
    end
    
    clfig(4)
    h=plot( lev, mul, 'g.');
    xlabel('level')
    ylabel('multiplicity')

    clfig(5)
    h=plot( mul, 'm.');
    xlabel('weight number')
    ylabel('multiplicity')
  end
  
  if Control.Verbose
    disp( ' ');
    disp( 'weight no   level   multiplicity  Dynkin index ');
    for i = 1:numwts
      disp( [sprintf( ['  %3d        %2d         %2d        [%2d' ...
        repmat( ',%2d', 1, rank-1) ']'], ...
        i, lev(i), mul(i), wt(i,:))])
    end
  end
  
end

% ------------------------------------------------------------------------
%
%
%
%
function Control = local_initcontrol( Control)

  if ~isfield( Control, 'Verbose')
    Control.Verbose = 1;
  end
  
  if ~isfield( Control, 'DoPlot')
    Control.DoPlot = 0;
  end
  
end

% ------------------------------------------------------------------------
%
%  check if Lie algebra of chosen type and rank actually exists.
%
function local_validateinput( type, rank)

  Fail = 0;

  switch upper( type)
    case 'A',      
      if rank < 1 
        Fail = 1;
      end
    case 'B',      
      if rank < 2  
        Fail = 1;
      end
    case 'C',      
      if rank < 3  
        Fail = 1;
      end
    case 'D',      
      if rank < 4  
        Fail = 1;
      end
    case 'E',      
      if rank < 6 || rank > 8
        Fail = 1;
      end
    case 'F',      
      if rank ~= 4 
        Fail = 1;
      end
    case 'G',      
      if rank ~= 2
        Fail = 1;
      end
    otherwise
      Fail = 1;
  end
  
  if Fail,
    error( [ upper( mfilename) ': Lie algebra >' ...
      upper( type) num2str( rank) '< does not exist.'])      
  end
  
end

% -------------------------------------------------------------------------
%
%  initialize
%
%  - Cartan matrix
%  - inverse of Cartan matrix
%  - metric
%  - dynkin index of adjoint representation
%    (weights of adjoint representation are the roots)
%
function [cartan, carinv, adjdynkin, metric, dimalg, detcar, ...
  m, mden] = local_initializevars( rank, type)
  
% Cartan matrix (inverted indices)
% NOTE: cartan(i,j) = TrueCartanMatrix(r-i+1,r-j+1)
%  cartan   = zeros( rank, rank, 'int32');
  
% inverse Cartan matrix (?)
  carinv   = zeros( rank, rank, 'int32');

% m(i,j) : Dynkin index of mu
%    m(i,j) ~ (alpha_i,alpha_j) 
%  m        = zeros( rank, rank, 'int32');

  adjdynkin = zeros( 1, rank, 'int32');
  
% metric 
%    metric(i,j) = (lambda_i,lambda_d) * d
%    i,j = 1,...,l  l = rank
%    lambda_i : basic weight
  metric = zeros( rank, rank, 'int32');

%
% simple root
%
%  alpha_i = sum_k=1^l lambda_k A(k,i)
%    A(k,i) : Cartan matrix
%    A(k,i) = 2 (alpha_i,alpha_j) / (alpha_i,alpha_i) 
  
%   for i = 1:rank,
%     m(i,i) = 2;
%   end
  m = int32( 2 * eye( rank));
  
%   for i = 1:rank,
%     for j = 1:rank,
%       if i == j
%         cartan(i,j) = 2;
%       elseif abs( i-j) == 1,
%         cartan(i,j) = -1;
%       end
%     end
%   end
  
%
%  (generic) Cartan matrix
%
  cartan = int32( 2 * eye( rank) + ...
                  diag( -1*ones( 1, rank-1), 1) + ...
                  diag( -1*ones( 1, rank-1), -1));

%
%  - dimension of Lie algebra
%  - determinant of Cartan matrix
%  - Dynkin index of adjoint representation
%  - initialize metric
%  - complete entries in Cartan matrix 'cartan'
%
  switch upper( type)

    case 'A',
      dimalg = rank * (rank + 2);
      detcar = rank + 1;
      mden   = 2 * (rank + 1);
      if rank == 1,
        adjdynkin(1)    = 2;
      else
        adjdynkin(1)    = 1;
        adjdynkin(rank) = 1;
      end
      for i = 1:rank,
%         for j = i:rank,
%           metric(i,j) = i * (rank+1-j);
%           metric(j,i) = metric(i,j); 
%         end
        metric(i,i:rank) = i * (rank+1-(i:rank));
        metric(i:rank,i) = metric(i,i:rank); 
      end
%       metric
%       keyboard     
      
    case 'B',
      dimalg = rank * (2 * rank + 1);
      detcar = 2;
      mden   = 4 * (2*rank - 1);
      cartan(1,2) = -2;
      for i = 2:rank,
        m(i,i) = 4;
      end
      if rank == 2
        adjdynkin(1) = 2;
      else
        adjdynkin(rank-1) = 1;
      end
      for i = 1:rank,
        for j = i:rank,
          if i == 1,
            metric(i,j) = (rank+1-j) * 2; 
          else
            metric(i,j) = (rank+1-j) * 4; 
          end
          metric(j,i) = metric(i,j); 
        end
      end
      metric(1,1) = rank; 
          
    case 'C',
      dimalg = rank * (2 * rank + 1);
      detcar = 2;
      mden   = 4 * (rank + 1);
      cartan(2,1) = -2;
      m(1,1) = 4;
      adjdynkin(rank) = 2;
      for i = 1:rank,
        for j = i:rank,
          metric(i,j) = 2 * (rank+1-j); 
          metric(j,i) = metric(i,j); 
        end
      end

    case 'D',
      dimalg = rank * (2 * rank - 1);
      detcar = 4;
      mden   = 4 * (rank - 1);
      cartan(1,2) = 0;
      cartan(2,1) = 0;
      cartan(1,3) = -1;
      cartan(3,1) = -1;
%      m(1,1) = 4;
      adjdynkin(rank-1) = 1;
      for i = 1:rank,
        for j = i:rank,
          if i <= 2,
            metric(i,j) = (rank+1-j) * 2; 
          else
            metric(i,j) = (rank+1-j) * 4; 
          end
          metric(j,i) = metric(i,j); 
        end
      end
      metric(1,1) = rank; 
      metric(2,2) = rank; 
      metric(1,2) = rank - 2; 
      metric(2,1) = rank - 2; 

    case 'E',      
      if rank == 6
        dimalg = 78;
      elseif rank == 7
        dimalg = 133;
      elseif rank == 8
        dimalg = 248;
      else
        error( [ upper( mfilename) ...
          ': Lie algebra >E' num2str( rank) '< does not exist.'])
      end
      detcar = 9 - rank;
      mden   = int32( 2) * ( idivide( dimalg, rank) - 1);
      cartan(1,2) = 0;
      cartan(2,1) = 0;
      cartan(1,4) = -1;
      cartan(4,1) = -1;
      
      if rank == 8,
        adjdynkin(rank) = 1;
      else
        adjdynkin(rank-5) = 1;
      end
      
      for i = 1:rank,
        for j = i:rank,
          if i == 1,
            metric(i,j) = (rank+1-j) * 3; 
          elseif i < 4,
            metric(i,j) = (rank+1-j) * (2*i-2); 
          else
            metric(i,j) = (rank+1-j) * (10-i); 
          end
          metric(j,i) = metric(i,j); 
        end
      end
      metric(1,1) = rank;
      metric(2,2) = 4; 
      metric(3,3) = 2 * rank - 2;
      metric(1,2) = rank - 3;
      metric(2,1) = metric(1,2);
      metric(1,3) = 2 * rank - 6;
      metric(3,1) = metric(1,3);
      metric(2,3) = rank - 1;
      metric(3,2) = metric(2,3);
      
    case 'F',
      dimalg = 52;
      detcar = 1;
      mden   = 36;
      cartan(3,2) = -2;
      m(1,1) = 4;
      m(2,2) = 4;
      adjdynkin(1) = 1;
      for i = 1:rank,
        for j = i:rank,
          if i == 1,
            metric(i,j) = (rank+1-j) * 2; 
          else
            metric(i,j) = (rank+1-j) * (6-i); 
          end
          metric(j,i) = metric(i,j); 
        end
      end
      metric(1,1) = 4; 
      
    case 'G',
      dimalg = 14;
      detcar = 1;
      mden   = 24;
      cartan(1,2) = -3;
      cartan(2,1) = -1;
      m(2,2) = 6;
      adjdynkin(2) = 1;
      metric(1,1) = 2; 
      metric(1,2) = 3; 
      metric(2,1) = 3; 
      metric(2,2) = 6; 
%      numroots = dimalg - rank + 1;
      
    otherwise
      error( [upper( mfilename) ': unknown Lie algebra type >' type '<'])
  end
 
  disp( [upper( mfilename) ': Lie algebra type / rank     = ' ...
    upper( type) num2str( rank) ])
  disp( [upper( mfilename) ': adj. dynkin indices         = [' ...
    num2str( adjdynkin) ']'])

end

% -------------------------------------------------------------------------
%
%
%
function local_testresults( root, cartan, metric, rank)

  metric  = double( metric);
  root    = double( root);
  cartan  = double( cartan);

% remove zero root
  ZeroIdx = (size( root, 2) + 1)/2;
  priv_assert( all( root(:,ZeroIdx) == 0))
  root(:,ZeroIdx) = [];

  NofRoots = size( root, 2);
  
  Tmp = zeros( rank, rank);
  
  for i = 1:rank,
    for j = 1:rank,
      jShft = j + NofRoots/2;
      iShft = i + NofRoots/2;
      Tmp(i,j) = 2 * (root(:,iShft)' * metric * root(:,jShft)) / ...
                     (root(:,iShft)' * metric * root(:,iShft));
    end
  end
  
  if max( max( abs( Tmp - cartan))) > 1e-6,
    Tmp
    cartan
    error( [upper( mfilename) ': inconsistent cartan matrix!'])
  end
  
%  priv_assert( Tmp == cartan)

  return
  
  Tmp
  cartan
  metric
  
  keyboard

end
