function [StructConst, AdjRep] = priv_structconstandadjrepr( metric, NCoeff)
%PRIV_STRUCTCONSTANDADJREPR   calculate structure constants and adj
%                             representation
%
%   [StructConst, AdjRep] = priv_structconstandadjrepr( metric, NCoeff)
%

% (c) G. Beyerle 2012

  global ROOTS
  global ROOTSRS
  global SUMOFROOTS
   
  NofRoots = size( ROOTS, 2);
  Rank     = size( ROOTS, 1); % dimension of Cartan subalgebra

%
% calculate fulls structure constants
%
  
  N = NofRoots + Rank;

  StructConst = zeros( N, N, N);

  NofPosRoots          = NofRoots / 2;
  SimpleRootsIdx       = NofPosRoots + (1:Rank);
  NotSimplePosRootsIdx = SimpleRootsIdx(end)+(1:(NofPosRoots-Rank));
  ZeroRootsIdx         = NofRoots + (1:Rank);

%  
%   f^k_{i,j} = StructConst(i,j,k)
%   f^k_{i,j} = -f^k_{j,i}
%

%
%  consider
%
%    [E_{beta}, E_{-beta}] = H_{beta}
%
%  a) beta+gamma is root
%
%    [E_{beta}, E_{gamma}] = N_{beta,gamma} * E_{beta+gamma} 
%
%  b) beta+gamma is not a root
%
%    [E_{beta}, E_{gamma}] = 0 
%
  for i = 1:NofRoots,
    for j = 1:NofRoots,
      k = SUMOFROOTS(i,j);
      if i + j == NofRoots+1,
% case [E_{beta}, E_{-beta}] is treated below        
        priv_assert( k == 0)
        continue
      end
      if k > 0,
        priv_assert( StructConst(i,j,k) == 0) 
        StructConst(i,j,k) = NCoeff(i,j);
      end
    end
  end

  
%
%  consider
%
%    [E_{beta}, E_{-beta}] = H_{beta}
%
%  case a) : beta is a simple root
%
%    => structure constant = +/-1
%
  for i = SimpleRootsIdx,
    k = i + NofRoots/2;  % index to H in cartan subalgebra
    jShft = NofRoots-i+1;
    priv_assert( StructConst(i,jShft,k) == 0) 
    StructConst(i,jShft,k) = 1;
    priv_assert( StructConst(jShft,i,k) == 0) 
    StructConst(jShft,i,k) = -1;
  end
  
%
%  consider
%
%    [E_{beta}, E_{-beta}] = H_{beta}
%
%  case b) : beta is _not_ a simple root
%

%
%  First, find coefficients of linear expansion of non-simple roots
%  in terms of simple roots (Wybourne, p.88, eqn. (8.4))
%
     
  RootCoeff = ROOTSRS;
  for i = SimpleRootsIdx,
    Idx = find( RootCoeff(:,i));
    priv_assert( length( Idx) == 1);
    priv_assert( RootCoeff(Idx,i) > 0 );
    RootCoeff(Idx,:) = RootCoeff(Idx,:) / RootCoeff(Idx,i);
  end
  
%   RootCoeff
%   keyboard
  
  for i = NotSimplePosRootsIdx,

    kFct = RootCoeff(end:-1:1,i); % reverse order in rank (first) coordinate!

    Denom = (ROOTS(:,i)' * metric * ROOTS(:,i));
    
    for l = 1:Rank,
      
%       Tmp = (ROOTS(:,l)' * metric * ROOTS(:,l)) / ...
%             (ROOTS(:,i)' * metric * ROOTS(:,i));
      Tmp = (ROOTS(:,l+NofPosRoots)' * metric * ROOTS(:,l+NofPosRoots)) / ...
            Denom;

      k = NofRoots+l;
      priv_assert( StructConst(i,NofRoots-i+1,k) == 0) 
      StructConst(i,NofRoots-i+1,k) = kFct(l) * Tmp;
% ... the corresponding (negative) value at the mirror location
      priv_assert( StructConst(NofRoots-i+1,i,k) == 0) 
      StructConst(NofRoots-i+1,i,k) = -kFct(l) * Tmp;

    end  
  end
  
%
%  consider
%
%    [H_{alpha}, E_{beta}]
%
%  and 
%
%    [E_{alpha}, H_{beta}] 
%

%   StructConst
%   keyboard

%   for i = 1:nofRoots
%     for j = 1:nofRoots
%       RootTimesRoot(i,j) = ROOTS(:,i)' * metric * ROOTS(:,j);
%     end
%   end

% roots
%
%  [-N, -N+1, ... -2, -1, 1, 2, R, R+1, ... N-1, N]
%
%  [1, 2, R]                   :  simple roots
%  [-N, -N+1, ... -2, -1]      :  negative roots
%  [1, 2, R, R+1, ... N-1, N]  :  positive roots

%
%  consider
%
%    [H_{beta}, E_{gamma}] = 2 (beta,gamma) / (beta,beta) E_{gamma}
%
%  if beta & gamma are simple roots, then
%
%    [H_{alpha_i}, E_{+/-alpha_j}] = +/-A_{i,j} E_{+/-alpha_j}
%
%  with +/-A_{i,j} element of Cartan matrix
%

  SavPos = zeros( Rank, Rank) * NaN;
  SavNeg = zeros( Rank, Rank) * NaN;

%   revROOTS  = ROOTS(end:-1:1,:);
%   revmetric = metric(end:-1:1,end:-1:1);
  
  RootXRoot = zeros( NofRoots, NofRoots);
  Tmp       = zeros( NofRoots, NofRoots);

  for i = 1:NofRoots,
    for j = 1:NofRoots,
      RootXRoot(i,j) = ROOTS(:,i)' * metric * ROOTS(:,j);
      Tmp(i,j)       = 2 * RootXRoot(i,j) / (ROOTS(:,i)' * metric * ROOTS(:,i));
    end
  end
  
%   Tmp
%   cartan(end:-1:1,end:-1:1)'
%   keyboard
  
%  Tmp = Tmp(:,end:-1:1);
%  Tmp = Tmp';
  
%  Tmp
%  keyboard
  
  for i = SimpleRootsIdx,
    for j = 1:NofRoots,

% %       Tmp = 2 * (ROOTS(:,j)' * metric * ROOTS(:,i)) / ...
% %                 (ROOTS(:,i)' * metric * ROOTS(:,i));


%       Tmp = 2 * (revROOTS(:,i)' * revmetric * revROOTS(:,j)) / ...
%                 (revROOTS(:,i)' * revmetric * revROOTS(:,i));
%       Tmp = 2 * (ROOTS(:,j)' * metric * ROOTS(:,i)) / ...
%                 (ROOTS(:,i)' * metric * ROOTS(:,i));
%       Tmp = 2 * (ROOTS(:,j)' * metric * ROOTS(:,i)) / ...
%                 (ROOTS(:,j)' * metric * ROOTS(:,j));
              
%      if j-NofRoots/2 >= 1 && j-NofRoots/2 <= Rank  

      if ~isempty( find( SimpleRootsIdx == j, 1))
        SavPos(i-NofPosRoots,j-NofPosRoots) = Tmp(i,j);    
      end
      if ~isempty( find( SimpleRootsIdx == NofRoots+1-j, 1))
        SavNeg(i-NofPosRoots,j-NofPosRoots+Rank) = Tmp(i,j);    
      end
                      
%       Tmp = 2 * (ROOTS(:,j)' * metric * ROOTS(:,i)) / ...
%                 (ROOTS(:,j)' * metric * ROOTS(:,j));
%       Tmp = 2 * (ROOTSRS(:,i)' * metric * ROOTSRS(:,j)) / ...
%                 (ROOTSRS(:,i)' * metric * ROOTSRS(:,i));              
%       if Tmp ~= Tmp2,
%         Tmp
%         Tmp2
%         keyboard
%       end

      k = j;

      iShft = i + NofPosRoots;

%       NofRoots
%       Rank
%       iShft

% reverse order of vectors in H (cartan subalgebra)
%      iShft = NofRoots + NofPosRoots + Rank + 1 - i;

%       iShft
%       keyboard
      
      %      iShft = (Rank+1-i) + NofPosRoots;
      priv_assert( StructConst(iShft,j,k) == 0) 
      StructConst(iShft,j,k) = Tmp(i,j); 
% structure constants are antisymmetric in first two indices
      priv_assert( StructConst(j,iShft,k) == 0) 
      StructConst(j,iShft,k) = -Tmp(i,j);      
    end
  end

%  priv_assert( SavPos == cartan)

%   SavPos
%   cartan(end:-1:1,end:-1:1)'
%   keyboard

%   if any( any( SavPos ~= cartan(end:-1:1,end:-1:1)'))
%     Tmp
%     SavPos
%     cartan(end:-1:1,end:-1:1)'
%     keyboard
%   end
  
%   SavPos
% %  SavNeg
%   cartan
%   keyboard
  
%   Tmp = NofRoots + (1:Rank);
%   StructConst(:,:,Tmp) = StructConst(:,:,Tmp(end:-1:1));      
%   StructConst(:,Tmp,:) = StructConst(:,Tmp(end:-1:1),:);
%   StructConst(Tmp,:,:) = StructConst(Tmp(end:-1:1),:,:);      

%
%  done filling entries in matrix 'StructConst' 
%
%   SavPos
%   SavNeg
%   cartan
%   keyboard
  
  Tmp = abs( StructConst - round( StructConst));
  MaxDevFromInteger = max( Tmp(:));
  priv_assert( MaxDevFromInteger < 1e-6);
  
  Tmp = abs( StructConst + permute( StructConst, [2 1 3]));
  MaxDevFromInteger = max( Tmp(:));
  priv_assert( MaxDevFromInteger < 1e-6);

  AdjRep = permute( StructConst, [3 2 1]); 

%   for k = 1:N,
%     % f^k_{i,j} -> (f_i)_{k,j}
%     % swap 2 and 3 index
%     disp( ['structure constants k = ' num2str( k) ' :'])
%     disp( StructConst(:,:,k))
%     disp(' ')
%   end
  
  
% %   ROOTS'
% %   keyboard
%   for k = 1:N,
%     % f^k_{i,j} -> (f_i)_{k,j}
%     % swap 2 and 3 index
%     disp( ['structure constants k = ' num2str( k) ' :'])
%     disp( StructConst(:,:,k))
%     disp(' ')
%   end
%   keyboard

%   for i = SimpleRootsIdx,
%     for j = 1:NofRoots,
% %       i
% %       i+NofRoots/2
%       mlassert( all( StructConst(j,i+NofRoots/2,:) == 0))
%       StructConst(j,i+NofRoots/2,:) = -StructConst(i+NofRoots/2,j,:);      
%     end
%   end
%  keyboard
  
%  StructConst
  
%  
%   g_{i,j} = \sum_{k,l} c^k_{i,l} * c^l_{j,k}
%  
  
%   g = zeros( N, N);
% 
%   for i = 1:N,
%     for j = 1:N,
%       for k = 1:N,
%         for l = 1:N,
%           g(i,j) = g(i,j) + StructConst(i,l,k) * StructConst(j,k,l);
%         end
%       end
%     end
%   end
    
%   for i = 1:N,
%     % f^k_{i,j} -> (f_i)_{k,j}
%     % swap 2 and 3 index
%     disp( ['generator X_' num2str( i) ' :'])
%     disp( squeeze( StructConst(i,:,:))')
%     disp(' ')
%   end
  
%
%  now loop thru all N^2 (N : diemnsion of Lie algebra) possibilities and 
%  check that the commutators are what we expect.
%

%
%  structure constants:
%
%   StructConst(i,j,k)  <->  f^k_{i,j}
%
% adjoint representation:
%
%   StructConst(i,j,k)  <->  adjrep^(i)_{k,j}
%
      
end

% % -------------------------------------------------------------------------
% %
% %  compare equality of two generators
% %
% function local_comparegenerators( Tmp1, Tmp2, i, j, k)
% 
%   if ~any( any( Tmp1 ~= Tmp2))
%     return
%   end
%  
%   disp( [ upper( mfilename) ': expected  [X_' num2str(i) ', X_' ...
%     num2str(j) '] = X_' num2str(k)])
%   disp( [ upper( mfilename) ': X_' num2str(k) ' ='])
%   disp( Tmp2)    
%   disp( [ upper( mfilename) ': but found [X_' num2str(i) ', X_' num2str(j) '] ='])
%   disp( Tmp1)    
%   keyboard
% 
% end
% 
% function local_validateCartanGenerator( Tmp, ZeroGenerator)
% 
%   NofGen = size( ZeroGenerator, 1);
%   for i = 1:NofGen
%     disp( [ upper( mfilename) ': H_' num2str(i) ' ='])
%     disp( squeeze( ZeroGenerator(i,:,:))')
%   end
%   Tmp
%   keyboard
%   
% end
