function [M1, M2, M3] = representationofso3( DimOfRepr, RealRepr)
%REPRESENTATIONOFSO3   n-dimensional representation of Lie algebra so(3)
%
%  [M1, M2, M3] = representationofso3( n, RealRepr)
%
%    DimOfRepr : dimension of representation matrices
%    RealRepr  : 1 = real representation wanted   
%

% (c) G. Beyerle 2013

  if nargin < 1,
    DimOfRepr = 2;
  end
    
  if nargin < 2,
    RealRepr = 0;
  end
  
  l = (DimOfRepr-1) / 2;
  
  if l == 0,
    return
  end
  
  if RealRepr,
    
    [M1, M2, M3] = local_realreprofso3( DimOfRepr);
                    
  else
    
    [M1, M2, M3] = local_complexreprofso3( DimOfRepr, l);
    
  end
  
  M1 = local_fix_rounding_errors( M1);
  M2 = local_fix_rounding_errors( M2);
  M3 = local_fix_rounding_errors( M3);
  
end

% -------------------------------------------------------------------------
%
%
%
%
function [M1, M2, M3] = local_complexreprofso3( n, l)

  priv_assert( length(-l:l) == n)

  %
  % representation matrices from QM angular momentum algebra
  %
  M3 = -1i * diag(-l:l);

  ll = (-l+1:l);
  Tmp1 = sqrt(l*(l+1)-(ll).*(ll-1));
  Tmp2 = sqrt(l*(l+1)-(ll-1).*(ll));
  
  M1 = -1i * 1/2  * ( diag( Tmp1, 1)  + diag( Tmp2, -1));
  M2 = -1i * 1/2i * ( diag( -Tmp1, 1) + diag( Tmp2, -1));
    
  % check that Lie algebra in fact obeys
  %
  %   [Mi, Mj] = eps_ijk Mk
  %
  Tmp = priv_liecommutator( M1, M2) - M3;
  priv_assert( max( max( abs( Tmp))) < 1e-6)

  Tmp = priv_liecommutator( M2, M3) - M1;
  priv_assert( max( max( abs( Tmp))) < 1e-6)

  Tmp = priv_liecommutator( M3, M1) - M2;
  priv_assert( max( max( abs( Tmp))) < 1e-6)

  priv_assert( priv_isskewhermitianmatrix( M1))
  priv_assert( priv_isskewhermitianmatrix( M2))
  priv_assert( priv_isskewhermitianmatrix( M3))

  M1 = local_fix_rounding_errors( M1);
  M2 = local_fix_rounding_errors( M2);
  M3 = local_fix_rounding_errors( M3);

  return
  
%   M1
%   M2
%   M3
%     
%   keyboard
    
end

% -------------------------------------------------------------------------
%
%
%
%
function M2 = local_fix_rounding_errors( M)
        
  MNew = sqrt( round( 2*M.^2) / 2) .* sign( M);
  Delta = abs( M - MNew);

%  max( max( Delta))
  
  if max( max( Delta)) < 1e-6
    M2 = MNew;
  else
    M2 = M;
  end
  
end

% -------------------------------------------------------------------------
%
% real representation of so(3)
%
% Refs:
%
%   V. M. Gordienko
%   Matrix entries of real representations of the group O(3) and SO(3)
%   Siberian Mathematical Journal (2002), 43(1):36-46
%   DOI: 10.1023/A:1013816403253
% 
%   G. Itzkowitz, S. Rothman and H. Strassberg
%   A note on the real representation of SU(2,C)
%   Journal of Pure and Applied Algebra (1991), 69(3):285-294
%   DOI: 10.1016/0022-4049(91)90023-U
%  
function [M1, M2, M3] = local_realreprofso3( l)

  if ( ~priv_isodd( l))
    error( [upper( mfilename) ...
      ': there is no ' num2str( l) ...
      '-dimensional representation of so(3).'])
  end
  
  priv_assert( priv_isodd( l))

  N = (l-1) / 2;
  
  h = zeros( 2*N+1, 2*N+1);
  
  k=1;
  for n = -N:N,
    
    % characterize homogeneous polynomial in x and y of degree 2N
    % P(x,y) = Tmp(1) x^{2N} + Tmp(2) x^{2N-1} y + ...
    %          + Tmp(2N) x y^{2N-1} + Tmp(2N+1) y^{2N} 
    Tmp = zeros(1,2*N+1);
    
    if n <= -1,
      Tmp(k)       = 1;
      Tmp(2*N+2-k) = -(-1)^n;
      h(k,:) = 1i^(N+1) * ...
               sqrt( factorial(2*N+1) / (2*factorial(N-n)*factorial(N+n))) * ...
               Tmp;
    elseif n >= +1
      Tmp(k)       = 1;
      Tmp(2*N+2-k) = (-1)^n;
      h(k,:) = (-1i)^(N) * ...
               sqrt( factorial(2*N+1) / (2*factorial(N+n)*factorial(N-n))) * ...
               Tmp;
    else         
      Tmp(k) = 1;
      h(k,:) = (1i)^(N) * ...
               sqrt( factorial(2*N+1)) / factorial(N) * ...
               Tmp;
    end
    
    k = k+1;
  end
  
%  h(k,:) are the polynomials h^(N)_n of Gordienko (p. 37)
  
%  h
  
%  h(1,:) = [a, b, c, d, e]
%  means: a X^4 + b X^3 Y + c X^2 Y^2 + d X Y^3 + e Y^4
  
% now we replace
%
%  X -> X' = conj(a) X - b Y
%  Y -> Y' = conj(b) X + a Y
% thus h(X,Y) -> h'(X,Y)
% objecvtive: express new basis h'(X,Y) in terms of old basis h(X.Y)
%
%   h'(X,Y) = W(a,b) h(X,Y)

%  Invh = inv( h);

  epsi = 1e-8;
  
  a1 = epsi;
  a2 = 0;
  a3 = 0;
  [a, b] = local_getunitarymatrix( a1, a2, a3);
  hP = local_calculate_hPrime( N, h, a, b);

  %  mlassert( priv_isorthogonalmatrix( hP * Invh))
  priv_assert( priv_isorthogonalmatrix( hP / h))
  
%  gx = (hP * Invh - eye(2*N+1)) / epsi;
  gx = (hP / h - eye(2*N+1)) / epsi;

  a1 = 0;
  a2 = epsi;
  a3 = 0;
  [a, b] = local_getunitarymatrix( a1, a2, a3);
  hP = local_calculate_hPrime( N, h, a, b);

  %  mlassert( priv_isorthogonalmatrix( hP * Invh))
  priv_assert( priv_isorthogonalmatrix( hP / h))
  
%  gy = (hP * Invh - eye(2*N+1)) / epsi;
  gy = (hP / h - eye(2*N+1)) / epsi;
  
  a1 = 0;
  a2 = 0;
  a3 = epsi;
  [a, b] = local_getunitarymatrix( a1, a2, a3);
  hP = local_calculate_hPrime( N, h, a, b);

  %  mlassert( priv_isorthogonalmatrix( hP * Invh))
  priv_assert( priv_isorthogonalmatrix( hP / h))
  
%  gz = (hP * Invh - eye(2*N+1)) / epsi;
  gz = (hP / h - eye(2*N+1)) / epsi;

%   g2x =  1/2 * gx
%   g2z =  1/2 * gy
%   g2y = -1/2 * gz

%   g2y =  1/2 * gx;
%   g2x =  1/2 * gy;
%   g2z = -1/2 * gz;

%   g2x =  1/2 * gx;
%   g2y =  1/2 * gy;
%   g2z =  1/2 * gz;

%
%  rearrange such that 3-dimensional representation matrices are
% N1 =
%      0     1     0
%     -1     0     0
%      0     0     0
% N2 =
%      0     0    -1
%      0     0     0
%      1     0     0
% N3 =
%      0     0     0
%      0     0     1
%      0    -1     0
%
  g2x =  1/2 * gx;
  g2y =  -1/2 * gz;
  g2z =  1/2 * gy;
%   g2x =  1/2 * gy;
%   g2y =  1/2 * gz;
%   g2z =  1/2 * gx;
  
  
  Thresh = 5000*epsi;
  
  priv_assert( priv_isskewsymmetricmatrix( g2x, Thresh))
  priv_assert( priv_isskewsymmetricmatrix( g2y, Thresh))
  priv_assert( priv_isskewsymmetricmatrix( g2z, Thresh))
  
  priv_assert( all( abs( priv_liecommutator( g2x, g2y) - g2z ) < Thresh ))
  priv_assert( all( abs( priv_liecommutator( g2y, g2z) - g2x ) < Thresh ))
  priv_assert( all( abs( priv_liecommutator( g2z, g2x) - g2y ) < Thresh ))  
  
  priv_assert( all( abs( imag( g2x)) < Thresh ))
  priv_assert( all( abs( imag( g2y)) < Thresh ))
  priv_assert( all( abs( imag( g2z)) < Thresh ))
  
  M1 = real( g2x);
  M2 = real( g2y);
  M3 = real( g2z);
      
end

% -------------------------------------------------------------------------
%
%  bestimme
%
%  a1 poly(pp1)^l poly(pp2)^0 + 
%  a2 poly(pp1)^(l-1) poly(pp2)^1 + 
%  a3 poly(pp1)^(l-1) poly(pp2)^2 + 
%             [...]
%  al poly(pp1)^1 poly(pp2)^(l-1) + 
%  a(l+1) poly(pp1)^0 poly(pp2)^l 
%
%  ak : binomial coefficient

function res = local_polynom( l, pp1, pp2)

  res = zeros(l+1,l+1);

  for i = 0:l,
        
    if i == 0,
      res(i+1,:) = local_polypower( pp1, l);
    elseif i == l,
      res(i+1,:) = local_polypower( pp2, l);
    else
      res(i+1,:) = conv( local_polypower( pp1, l-i), ...
                         local_polypower( pp2, i));
    end
    
  end

end

% -------------------------------------------------------------------------
% 
%   pp3 = conv( pp1, pp2);
% 
% %   Tmp = kron( flipdim( diag( pp1), 1), flipdim( diag( pp2), 1));
% %   pp3 = diag( flipdim( Tmp, 1));
% 
function pp2 = local_polypower( pp, n)

%  mlassert( isinteger( n))

  priv_assert( n > 0)
  
  if n == 1
    pp2 = pp;
    return
  end
  
%   ppF = flipdim( diag( pp), 1);
%   Tmp = ppF;

  Tmp = pp;

  for i = 2:n
%    Tmp = kron( Tmp, ppF);
    Tmp = conv( Tmp, pp);
  end
  
%  pp2 = diag( flipdim( Tmp, 1))';
  pp2 = Tmp;

end

% ------------------------------------------------------------------------- 
%
%  h = h(1) x^n y^0 + h(2) x^{n-1} y^1 + ... + h(n+1) x^0 y^n 
%
%  x -> xP(1) x + xP(2) * y
%  y -> yP(1) x + yP(2) * y
%
function hP = local_replacepolynom( h, xP, yP)
  
  hP = zeros( size( h));
  
  hLst = find( abs( h));

%   h
%   hLst
%   keyboard
  
  N = (length( h) - 1) / 2;
  
  for i = hLst,
        
    if i-1 == 0 && 2*N+1-i >= 1,
      Tmp = priv_polypower( xP, 2*N+1-i);
    elseif i-1 >= 1 && 2*N+1-i == 0,
      Tmp = priv_polypower( yP, i-1);
    elseif i-1 >= 1 && 2*N+1-i >= 1,
      Tmp = conv( priv_polypower( xP, 2*N+1-i), ... 
                  priv_polypower( yP, i-1));
    else
      error( [upper( mfilename) ': i = ' num2str( i) ', N = ' num2str( N)])
    end
    
%     i
%     Tmp
    
    hP = hP + h(i) * Tmp;
    
  end
     
  return
  
  xP
  yP
  h
  hP
  keyboard

end

% -------------------------------------------------------------------------
%
%
%
%
%
function  hP = local_calculate_hPrime( N, h, a, b)

  hP = zeros( 2*N+1, 2*N+1);

  for k = 1:2*N+1,

%    n = k-N-1;
    
    hP(k,:) = local_replacepolynom( h(k,:), ...
      [conj(a), -b], [conj(b), a]);
    
  end
  
end

% -------------------------------------------------------------------------
%
%  unitary 2x2 matrix parameterized by 3 variable a1, a2, a3
%
%  Q = [ conj(a)   -b ]
%      [ conj(b)    a ]
%
%  X -> X' = conj(a) X - b Y
%  Y -> Y' = conj(b) X + a Y
%
%  (X' Y') = (X Y) * Q
%
function [a, b] = local_getunitarymatrix( a1, a2, a3)

%   Q = [exp(1i*(w+ph)/2)*cos(th/2),     1i*exp(1i*(w-ph)/2)*sin(th/2);
%        1i*exp(-1i*(w-ph)/2)*sin(th/2), exp(-1i*(w+ph)/2)*cos(th/2)];  

%
%
%
%   a = Q(1,1);  
%   b = Q(1,2);  

  a0 = sqrt(1 - a1^2 - a2^2 - a3^2);

  a = a0 + 1i * a3;
  b = a2 + 1i * a1;

%  Q = [ conj(a)   conj(b) ]
%      [   -b         a    ]
%    = [   a0 - 1i * a3 ,   a2 - 1i * a1 ]
%      [ - a2 - 1i * a1 ,   a0 + 1i * a3 ]
% with a0 = sqrt(1 - a1^2 - a2^2 - a3^2)
  
  priv_assert( abs( a * conj(a) + b * conj(b) - 1) < 1e-12)

end
